// this is our simulation class

public class Simulation extends Object
{
    private EventList myMaster; // this is the main eventList, that is used by everything
    /*
      pre: none
      post: constructs a simulation class in memory
    */
    public Simulation()
    {
    }

    //pre:  ssPercent + fcPercent + ePercent = 1, s1Percent + s2Percent = 1
    //      ssPercent, fcPercent, ePercent, s1Percent, s2Percent, cPercent > 0
    //      ssPercent, fcPercent, ePercent, s1Percent, s2Percent, cPercent are accurate to 2 or fewer decimal places
    //      meanArrival, meanSS, meanFC, meanE, meanC, meanS > 0
    //post: returns a basket of statistics computed by one run of this
    //      simulation using seedA, seedSS, seedF, seedE, seedC, seedS1, seedS2, seedCheckIn, seedCustom, seedSec
    //      to initialize the pseudorandom number generators



    public StatsBasket runSimulation(long seedA, long seedSS, long seedF, long seedE, long seedC, long seedS1, long seedS2,long seedCheckIn, long seedCustom, long seedSec,double meanArrival, double meanSS, double meanFC, double meanE, double meanC, double meanS,double ssPercent, double fcPercent, double ePercent, double cPercent, double s1Percent, double s2Percent)
    {
      this.myMaster = new EventList(); // creates an event list

      Server sec1 = new Security(null,Constants.SEC_NAMES[0],myMaster,seedS1,meanS); // creates security 1
      Server sec2 = new Security(null,Constants.SEC_NAMES[1],myMaster,seedS2,meanS); // creates security 2


      double[] probs = new double[2]; // creates the probabilities of going to each security

      Server[] linkSec = new Server[2]; // creates an array of references to the securities
      linkSec[0] = sec1; // fills up array, pt.1
      linkSec[1] = sec2; // fills up array, pt.2
      Server customs = new PreSecurity(linkSec,"Customs",myMaster,seedC,meanC); // creates a customs object, that links to the 2 securities

      Server [] checkins = new CheckIn[3]; // creates an array of check in counters
      Server[] linkCheck = new Server[3]; // creates the links that each check in counter has
      linkCheck[0] = linkSec[0]; // fills link, pt.1
      linkCheck[1] = linkSec[1]; // files link, pt.2
      linkCheck[2] = customs; // fills link, pt.3

      checkins[0] = new CheckIn(linkCheck,"Self-Service",myMaster,seedSS,meanSS,2); // creates the self service counter
      checkins[1] = new CheckIn(linkCheck,"First Class",myMaster,seedF,meanFC,2); // creates the first class counter
      checkins[2] = new CheckIn(linkCheck,"Economy",myMaster,seedE,meanE,2); // creates the economy class counter

      probs = new double[3]; // creates the probabilities of going to each of the check in counters
      probs[0] = ssPercent;
      probs[1] = fcPercent; // fills the array up
      probs[2] = ePercent;
      // constructs the entrance
      PeopleMaker people = new PeopleMaker(probs,checkins,"Arrival",myMaster,seedCheckIn,meanArrival,
                                      seedSec,seedCustom,cPercent,seedA,s1Percent,s2Percent);
      people.Run(0); // creates a single event in the eventList, which will begin the chain reaction



      int curTime=0; // starts time out at zero
      while(!people.Done()) // as long as people maker has people to spew...
      { curTime = this.myMaster.getNextEventTime(); // the current time is the time of the next event
        Event curEvent = this.myMaster.getNextEvent(curTime); // gets the next event occuring at curTime

        while(curEvent != null && !people.Done()) // as long as the last event we grabbed was valid... (if there are no more for this time, it will be null)
        {
          curEvent.getServer().Run(curTime); // runs the event on the server specified

          curEvent= this.myMaster.getNextEvent(curTime); // grabs the next event at time curTime

        }
      }

      StatsHolder SSstats = checkins[0].getStats();
      StatsHolder Cstats = customs.getStats(); // grabs the stats out of each of the things
      StatsHolder S1stats = sec1.getStats();
      

      return new StatsBasket(SSstats.percentUse(), Cstats.percentUse(), S1stats.percentUse(),
                              SSstats.maxLength(), Cstats.maxLength(), S1stats.maxLength (),
                              SSstats.avgLength(), Cstats.avgLength(), S1stats.avgLength(),
                              StatsHolder.maxTime(), StatsHolder.avgTime());

    }
}
