public class StatsHolder extends Object
{ private static Queue timeSpent = new Queue();
  private static int numPeople; //Number of people in the system
  private static int numLeft; //Number of people that have left

  private Queue qLength;
  private Queue qInUse;
  private String serverName;

  public StatsHolder(String sName)
  { // pre: true
    // post: Creates a new StatsHolder object, used by a server to collect statistics
    this.serverName = sName;
    qLength = new Queue();
    qInUse = new Queue();
  }

  public void addLength(int length, int time)
  { // pre: length is not null, time is not null and later then any previous times
    // post: add a Length element to the Length Queue
    if (numPeople >= 100 && numLeft <= 1900) qLength.push (new LengthOfQ(length,time));
  }

  public void addUse(boolean inUse, int time)
  { // pre: inUse is not null, time is not null and later then any previous times
    // post: add a Use element to the Use Queue
    if (numPeople >= 100 && numLeft <= 1900) qInUse.push (new UseOfQ(inUse,time));
  }

  public static void fullTime(int startTime, int endTime)
  { // pre: startTime is not null, endTime is not null and greater then startTime
    // post: add an Integer element to the time spent in simulation Queue
    if (numPeople >= 100 && numLeft <= 1900)
    { //add a person's full stay to the static Time queue
      Integer time = new Integer(endTime - startTime);
      timeSpent.push(time);
    }
  }

  public static void setPeople(int num)
  { // pre: num is not null
    // post: set the number of people that have entered the simulation to num
    numPeople = num; //num people in the system
  }

  public static int getPeople()
  { // pre: true
    // post: returns the number of people that have entered the system
    return numPeople;
  }

  public static void setLeft()
  { // pre: true
    // post: increment the number of people that have left the system (passed security)
    numLeft ++; //num people who have left the system
  }

  public int maxLength()
  { // pre: No entry occured at time -100
    // post: Return the maximum length stored in Length Queue
    int max = 0;
    LengthOfQ temp;

    qLength.push(new LengthOfQ(-100,-100)); //Place holder at end of queue
    //loop through until and store the largest number
    while(((LengthOfQ)qLength.peek()).getTime() != -100 )
    { temp = (LengthOfQ)qLength.pop();
      if (temp.getLength() > max) max = temp.getLength();
      qLength.push(temp); //iterate through
    }
    qLength.pop(); //remove placeholder

    return max;
  }

  public double avgLength()
  { //pre: no entry occured at time -100.  all entries are ordered in length queue chronological order
    //post: Return weighted average length from stats in Length Queue
    double length = 0;
    LengthOfQ temp;
    int startTime = 0;
    int lastTime = 0;

    qLength.push(new LengthOfQ(-100,-100)); //Place holder at end of queue
    startTime = ((LengthOfQ)qLength.peek()).getTime();

    //loop through taking weighted average
    while(((LengthOfQ)qLength.peek()).getTime() != -100 )
    { temp = (LengthOfQ)qLength.pop();
      if (((LengthOfQ)qLength.peek()).getTime() != -100)
      /** if its not the last one (because that is used only for the upper time limit)
          take:: length * (next time - its time)
      */
      { lastTime = ((LengthOfQ)qLength.peek()).getTime();
        length = length + temp.getLength() * (lastTime - temp.getTime());
      }
      qLength.push(temp); //iterate through
    }
    qLength.pop(); //remove place holder

    return length / (lastTime - startTime);
  }

  public double percentUse()
  { //pre: All entries are ordered in length queue chronological order
    //Post: Return the percent of time the queue was in use
    int startTime=0;
    int curTime = 0;
    int endTime = 1;
    int useTime = 0;
    UseOfQ use;
    startTime = ((UseOfQ)qInUse.peek()).getTime();

    // iterate through, adding the sum of time lengths when inUse is true, and divide the sum by the total time
    while(!qInUse.isEmpty())
    { use = (UseOfQ)qInUse.pop();
      curTime = use.getTime(); //get the time of this usage

      if (!qInUse.isEmpty())
      { endTime = ((UseOfQ)qInUse.peek()).getTime(); //time of next usage
        if(use.getInUse())
        { useTime = useTime + (endTime - curTime);
        }
      }
    }
    //return the time in use divided by the total time
    return (double)(useTime / (double)(endTime - startTime));
  }

  public static int maxTime()
  { //Post: Return the longest time anoyone spent in the terminal
    int max = 0;
    int temp;

    timeSpent.push(new Integer(-1)); //Place holder at end of queue
    //loop through and store the largest total time
    while(((Integer)timeSpent.peek()).getInt() != -1 )
    { temp = ((Integer)timeSpent.peek()).getInt();
      if (temp > max) max = temp;
      timeSpent.push(timeSpent.pop()); //iterate through
    }
    

    timeSpent.pop(); //remove placeholder
    return max; //return the maximum value
  }

  public static double avgTime()
  { //Post: Return the average time spent in the terminal
    int temp = 0;

    timeSpent.push(new Integer(-100)); //Place holder at end of queue

    while(((Integer)timeSpent.peek()).getInt() != -100 )
    { temp = temp + ((Integer)timeSpent.peek()).getInt(); //add the time
      timeSpent.push(timeSpent.pop()); //iterate through
    }
    timeSpent.pop(); //remove place holder

    temp = temp / timeSpent.size(); //divide by total to get average
    return temp;
  }

 //Stats testing area
 /* public static void main(String[] args)
  { StatsHolder a = new StatsHolder("null");
    a.setPeople(500);
    a.addLength (1, 1);
    a.addLength (4,18);
    a.addLength (20,19);
    a.addLength (5,20);
    d.w(a.avgLength ()+"::" + a.maxLength ());

    a.addUse (true,10);
    a.addUse(false,15);
    a.addUse(false,20);
    a.addUse(true,30);
    d.w(a.percentUse () +"");

    a.fullTime (0,30);
    a.fullTime(0,10);
    a.fullTime(0,80);
    a.fullTime(10,10);
    d.w(a.avgTime ()  +"::"+ a.maxTime ());
  } */


}