public class Random {

  /* A class used to create pseudorandom numbers. 

     To use this class, first construct a pseudorandom number generator
     by using on of the constructors. The two constructors differ
     in how they select a seed. A seed is a number used to generate 
     a certain sequence of pseudorandom numbers. Two different pseudorandom 
     number generators constructed using the same seed will yield 
     the same sequence of numbers. It is not the upper or lower
     bound nor the expected value or mean.
  */

  private java.util.Random r; // the pseudorandom number generator

  public Random () {
  //post: returns a new random number generator based on the current time
    r = new java.util.Random(); // initializes r based on the current time
  }

  public Random (long seed) {
  //post: returns a new pseudorandom number generator using seed
    r = new java.util.Random(seed);
  }

  public double nextExp (double alpha) {
  //pre: alpha > 0
  //post: returns a pseudorandom number selected from the exponential
  //      distribution with expected value (or mean) alpha
    double x;
    while ((x=r.nextDouble()) == 0); //loop until x!=0
    return (- alpha * Math.log(x));
  }

  public int nextUniformInt (int low, int high) {
  //post: returns a pseudorandom int uniformly distributed between 
  //      low and and high
    double x;
    while ((x=r.nextDouble()) == 1); //loop until x!=1;
    return (int) Math.floor(x * (high - low + 1) + low);
  }
}


