// the server is our base class for "A thing that interacts with people and causes events"

public abstract class Server extends Object
{
  protected Server[] linkedTo; // the servers that follow this one

  private Queue myQ; // the queue of people lined up for this server
  private String myName; // name of this server, such as security one, or something like that
  protected EventList eventMaster; // the master of all events
  private double meanTime; // mean amount of time this takes to deal with people
  private Random myTimeRandom; // for random timing
  private Person user; // the person that is "at the front desk"

  protected StatsHolder stats; // for storing statistics

  /*
    pre: linkedTo is not null, master is not null
    post: creates a server
  */
  public Server(Server[] linkedTo,String myName, EventList master, long seederTime,double meanTime)
  {
    this.myTimeRandom = new Random(seederTime);
    myQ = new Queue();
    this.myName = myName;
    this.meanTime = meanTime;
    this.linkedTo = linkedTo;
    this.eventMaster = master;
    user = null;

    stats = new StatsHolder(myName);
  }
  /*
    pre: none
    post: returns the name of this server
  */
  public String getName()
  { return this.myName;
  }

  /*
    pre: who is not null
    post: adds a person to the queue of this server.  If there was nobody here before, that person
          is sent to the "front desk", and a new event is added
  */
  public void addPerson(Person who,int currentTime)
  {
    this.myQ.push(who);
    //d.w("Recieved by " + this.myName + ".  Size = " + this.myQ.size());
    if(this.free()) // if the new person is at the front of the line already, they need a new event made for them
    {  this.user = (Person)this.myQ.pop();
       this.setNewEvent(currentTime);
       //this.Run(currentTime); // this will add the person to the server, and add a new event
       stats.addUse(true, currentTime); //add usage time
    }

    stats.addLength(myQ.size(),currentTime); //Add current time, and size of the queue, measure until next time
  }
  /*
    pre: none
    post: adds a new event that will occur at currentTime + some random number
  */
  protected void setNewEvent(int currentTime)
  { double timeSpent = this.myTimeRandom.nextExp(this.meanTime); // how long this customer spends here
    int eventTime = currentTime + (int)(timeSpent); // when they will be done
    //d.w("Next event will occur at " + eventTime);
    this.eventMaster.addEvent(eventTime,this);
  }


  /*
    pre: none
    post: the person at the front desk has been dispatched.  If there was a person after them, then
          a new event has been added for them, and they are now at the front desk
  */
  public void Run(int currentTime)
  { // if run is called, that means an event occured at this server


    if (user != null) // if we have someone here.  This should always be true
    { //d.w(this.getName() + " Sending user to next server");
      this.sendToLink(user,currentTime); //send user to next server
      stats.addUse(false, currentTime); // collect stats
    }

    if ( myQ.size() > 0 ) // if there are people left in the queue
    { //d.w(this.getName() + " adding person to front desk variable");
      user = (Person)this.myQ.pop(); //store user in server
      stats.addLength (this.myQ.size(), currentTime); //add length time
      stats.addUse(true, currentTime); //add usage time
      this.setNewEvent(currentTime); // adds a new event for this person to be done
    }
    else // if no-one is in the queue, just clear out the front desk
    { this.user = null;
    }

  }


  abstract void sendToLink(Person who,int currentTime); // must be implemented by the various different server types



  /*
    pre: none
    post: returns if this server is serving anyone at the moment
  */
  private boolean free()
  { return (this.user == null);
  }
  /*
    pre: none
    post: returns the statsHolder class contained within this server
  */
  public StatsHolder getStats()
  { return stats;
  }

}